package es.verdnatura.presentation.view.feature.historicoshelvinglog.fragment

import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import com.google.gson.Gson
import es.verdnatura.R
import es.verdnatura.databinding.FragmentShelvinglogBinding
import es.verdnatura.domain.ConstAndValues.BASEURLSALIX
import es.verdnatura.domain.notNull
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.historicoshelvinglog.adapter.ShelvingLogAdapter
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO

@Suppress("UNUSED_ANONYMOUS_PARAMETER")
class ShelvingLogFragment(
    var entryPoint: String = ""

) : BaseFragment<FragmentShelvinglogBinding, ShelvingLogViewModel>(
    ShelvingLogViewModel::class
) {
    private var adapter: ShelvingLogAdapter? = null
    private var itemScan = ""

    companion object {
        fun newInstance(entryPoint: String) = ShelvingLogFragment(entryPoint)
    }

    private lateinit var customDialog: CustomDialog

    override fun getLayoutId(): Int = R.layout.fragment_shelvinglog

    override fun init() {

        customDialog = CustomDialog(requireContext())
        binding.linearHeader.visibility = View.GONE
        setEvents()
        setToolBar(getString(R.string.shelvingLog))
        super.init()

    }

    private fun setToolBar(title: String) {
        binding.mainToolbar.toolbarTitle.text = title

    }

    private fun setEvents() {
        binding.scanInput.requestFocus()
        binding.scanInput.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!binding.scanInput.text.toString().isNullOrEmpty()) {
                    viewModel.shelvings((binding.scanInput.text.toString()))
                    itemScan = binding.scanInput.text.toString()
                }
                binding.scanInput.setText("")
                ma.hideKeyboard(binding.scanInput)
                return@setOnEditorActionListener true
            }
            false
        }

        binding.mainToolbar.backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

    }

    override fun observeViewModel() {
        with(viewModel) {

            loadShelvingLogSalixList.observe(viewLifecycleOwner, Observer { event ->
                event.getContentIfNotHandled().notNull {
                    binding.itemshelvinglogRecyclerview.adapter = adapter
                    binding.itemshelvinglogRecyclerview.layoutManager =
                        LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
                    setToolBar(getString(R.string.shelvingLog) + ":" + itemScan)
                    if (it.list.isEmpty()) {
                        customDialog.setTitle(getString(R.string.noResults))
                            .setDescription(getString(R.string.noDataLabelScanned))
                            .setOkButton(getString(R.string.close)) {
                                customDialog.dismiss()
                            }.show()
                    } else {
                        val itemId = it.list[0].id
                        ma.onPasillerosItemClickListener(
                            PasillerosItemVO(
                                title = getString(R.string.titleWebViewer),
                            ),
                            entryPoint = Gson().toJson(
                                mutableMapOf(
                                    "entryPoint" to itemId,
                                    "web" to "${mobileApplication.dataStoreApp.readDataStoreKey<String>(
                                        BASEURLSALIX
                                    )}/#!/shelving/$itemId/log"
                                )
                            )
                        )
                    }
                }

            })

        }
    }

}
